from typing import Literal, TypedDict, NotRequired, Any

__all__ = [
    "ErrorResult",
    "Error",
    "ErrorAttributes",
    "ErrorCodeType",
]

ErrorCodeType = Literal[
    "ACCOUNT_ALREADY_JOINED_SITE",
    "ALREADY_PRIMARY_ENVIRONMENT",
    "CANNOT_CREATE_OR_DESTROY_SCHEMA_MENU_ITEM_LINKED_TO_ITEM_TYPE",
    "CANNOT_DESTROY_CURRENT_USER",
    "CANNOT_DESTROY_FORKING_ORIGIN",
    "CANNOT_DESTROY_PRIMARY_ENVIRONMENT",
    "CANNOT_UPGRADE_MODERN_PLUGIN_INTO_LEGACY",
    "CLASHING_FIELD_LABELS",
    "CONCURRENT_ENVIRONMENT_UPDATE",
    "CONCURRENT_ITEM_TYPE_UPDATE",
    "CONCURRENT_ITEM_UPDATE",
    "CONCURRENT_ROLE_UPDATE",
    "CONCURRENT_UPLOAD_UPDATE",
    "DEACTIVATED_SITE",
    "DELETE_RESTRICTION",
    "DESTINATION_USER_REQUIRED",
    "DUPLICATE_POSITIONS_FOUND",
    "DUPLICATE_SINGLETON",
    "ENVIRONMENT_IN_READ_ONLY_MODE",
    "ENVIRONMENT_NOT_READY",
    "EXCEPTION",
    "IMMUTABLE_UPLOAD_TRACKS_IN_SANDBOX_ENVIRONMENT",
    "INCOMPATIBLE_WITH_UPLOAD_STORAGE_SETTINGS",
    "INSUFFICIENT_PERMISSIONS",
    "INVALID_ACCEPT_HEADER",
    "INVALID_API_VERSION",
    "INVALID_ATTRIBUTES",
    "INVALID_AUTHORIZATION_HEADER",
    "INVALID_CONTENT_TYPE_HEADER",
    "INVALID_DATE",
    "INVALID_DESTINATION_TYPE",
    "INVALID_DESTINATION_USER",
    "INVALID_DRAFT",
    "INVALID_ENDPOINT",
    "INVALID_ENTITY_ID",
    "INVALID_ENVIRONMENT",
    "INVALID_FIELD",
    "INVALID_FILTER_FIELDS_PARAM",
    "INVALID_FORMAT",
    "INVALID_JSON_BODY",
    "INVALID_ORDERING",
    "INVALID_ORDERING_FOR_SINGLETON_ITEM_TYPE",
    "INVALID_ORDERING_FOR_SORTABLE_ITEM_TYPE",
    "INVALID_PARAMS",
    "INVALID_PARENT",
    "INVALID_PARENT_ID",
    "INVALID_PLUGIN_VERSION",
    "INVALID_POSITION",
    "INVALID_RELATIONSHIP",
    "INVALID_REQUEST",
    "INVALID_SITE",
    "INVALID_SITE_EXPORT_SETTINGS",
    "INVALID_TYPE",
    "INVALID_UPLOAD_STORAGE_SETTINGS",
    "ITEM_LOCKED",
    "ITEM_TYPE_CANNOT_BE_CHANGED",
    "ITEM_TYPE_IS_SINGLETON",
    "ITEM_TYPE_NOT_FOUND",
    "KIND_CANNOT_BE_CHANGED",
    "MAINTENANCE_MODE",
    "MISSING_BUILD_TRIGGER_ID_PARAMETER",
    "MISSING_FIELDS",
    "MISSING_LOCALES",
    "MISSING_QUERY_PARAMETER",
    "MISSING_UPLOAD_IDS",
    "MODULAR_BLOCK_IN_USE",
    "MUX_ERROR",
    "NEW_PLUGIN_VERSION_IS_INCOMPATIBLE",
    "NON_EDITABLE_ACCESS_TOKEN",
    "NOT_A_VIDEO",
    "NOT_FOUND",
    "NOT_ON_PER_SITE_PRICING",
    "NO_PRIMARY_AUDIO_TRACK",
    "PLAN_UPGRADE_REQUIRED",
    "PLATFORM_SCHEDULED_MAINTENANCE",
    "PRIMARY_ENVIRONMENT_SETTINGS_READ_ONLY",
    "PUBLISHED_CHILDREN",
    "PUBLISHED_REFERENCES",
    "RATE_LIMIT_EXCEEDED",
    "REQUIRED_BY_ASSOCIATION",
    "SERVICE_UNAVAILABLE",
    "SITE_NOT_READY",
    "SSO_SETTINGS_REQUIRED",
    "STALE_ITEM_VERSION",
    "TECHNICAL_LIMIT_REACHED",
    "TOO_MANY_OPERATIONS",
    "UNMANAGED_EDIT_CONFLICT",
    "UNPUBLISHED_LINK",
    "UNPUBLISHED_PARENT",
    "UPLOAD_IS_CURRENTLY_IN_USE",
    "UPLOAD_NOT_PASSING_FIELD_VALIDATIONS",
    "USED_AS_SLUG_SOURCE",
]


class ErrorAttributes(TypedDict):
    code: ErrorCodeType
    doc_url: str
    transient: NotRequired[bool]
    details: dict[str, Any]


class Error(TypedDict):
    id: str
    type: Literal["api_error"]
    attributes: ErrorAttributes


class ErrorResult(TypedDict):
    data: list[Error]
